"""
Command-line capability launcher

Sends a request for a capability execution to the capability service
"""
import json
import argparse
import sys

import requests
from channels.amqp_helpers import Channel, CapabilityEventChannel
from pycapo import CapoConfig

from workspaces.schema import Capability, CapabilityEvent
from ._version import ___version___ as version


def request_capability():
    """
    Send HTTP request to capability service and accept a return request
    :return:
    """
    pass


def _make_arg_parser() -> argparse.ArgumentParser:
    """
    Create argument parser for command-line arguments
    :return: An argument parser object
    """
    description = (
        f"Command-line capability launcher v{version}. For now, this is used to launch the"
        f"null capability. In the future, capabilities will be launched from a web UI and"
        f"stored in the archive."
    )
    parser = argparse.ArgumentParser(
        description=description, formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument("capability", action='store', nargs=1,
                        help='Name of the capability to run')

    return parser


def receive_capability_event(event: CapabilityEvent):
    print(event)
    if event.complete:
        sys.exit(0)


def main():
    args = _make_arg_parser().parse_args()

    settings = CapoConfig().settings('edu.nrao.archive.workspaces.CapabilitySettings')

    # Submit HTTP capability request to server
    r = requests.post(f"{settings.serviceUrl}/capability/request",
                      json={'capability': args.capability})

    # we should get back a request ID; if we do, submit it and obtain
    # an execution ID
    request_id = r.json()['id']
    r = requests.post(f'{settings.serviceUrl}/capability/request/{request_id}/submit')
    print(r.content)
    execution_id = r.json()['id']

    # at this point we can start listening for events from rabbitmq concerning
    # this execution
    with Channel(CapabilityEventChannel()) as channel:
        channel.listen(receive_capability_event,
                       pattern=f'request-{request_id}.execution-{execution_id}.#')


if __name__ == "__main__":
    main()
